# set working directory (see https://stackoverflow.com/a/35842119)
dir = tryCatch({
  # script being sourced
  getSrcDirectory()[1]
}, error = function(e) {
  # script being run in RStudio
  dirname(rstudioapi::getActiveDocumentContext()$path)
})
setwd(dir)

source("../global.R")

library(data.table)


### java_regex ###

PREFIX <- "java_regex"

# read sample of threads without links
threads_without_links <- read.csv(file=paste0(PREFIX, "/threads_without_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_without_links)
# 100
length(unique(threads_without_links$PostId))
# 100

# read sample of threads with documentation links
threads_with_documentation_links <- read.csv(file=paste0(PREFIX, "/threads_with_documentation_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_with_documentation_links)
# 102
length(unique(threads_with_documentation_links$PostId))
# 102

# read sample of threads without documentation links
threads_without_documentation_links <- read.csv(file=paste0(PREFIX, "/threads_without_documentation_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_without_documentation_links)
# 102
length(unique(threads_without_documentation_links$PostId))
# 102

question_metadata <- retrieve_metadata()
question_post_blocks <- retrieve_postblocks()

# retrieve metadata for samples

# threads without links
threads_without_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_without_links$PostId,]
nrow(threads_without_links_question_metadata)
# 100
incomplete_cases <- which(is.na(threads_without_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 0
threads_without_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_without_links_question_metadata$PostId,]
nrow(threads_without_links_question_post_blocks)
# 406
# export data
write.table(threads_without_links_question_metadata, file=paste0(PREFIX, "/threads_without_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_without_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_without_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# threads with documentation links
threads_with_documentation_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_with_documentation_links$PostId,]
nrow(threads_with_documentation_links_question_metadata)
# 102
incomplete_cases <- which(is.na(threads_with_documentation_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 2
threads_with_documentation_links_question_metadata <- threads_with_documentation_links_question_metadata[-incomplete_cases,]
nrow(threads_with_documentation_links_question_metadata)
# 100
threads_with_documentation_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_with_documentation_links_question_metadata$PostId,]
nrow(threads_with_documentation_links_question_post_blocks)
# 393
# export data
write.table(threads_with_documentation_links_question_metadata, file=paste0(PREFIX, "/threads_with_documentation_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_with_documentation_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_with_documentation_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# threads without documentation links
threads_without_documentation_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_without_documentation_links$PostId,]
nrow(threads_without_documentation_links_question_metadata)
# 102
incomplete_cases <- which(is.na(threads_without_documentation_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 2
threads_without_documentation_links_question_metadata <- threads_without_documentation_links_question_metadata[-incomplete_cases,]
nrow(threads_without_documentation_links_question_metadata)
# 100
threads_without_documentation_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_without_documentation_links_question_metadata$PostId,]
nrow(threads_without_documentation_links_question_post_blocks)
# 387
# export data
write.table(threads_without_documentation_links_question_metadata, file=paste0(PREFIX, "/threads_without_documentation_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_without_documentation_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_without_documentation_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# merge metadata of all three samples 
threads_without_links_question_metadata$Category <- rep(0, nrow(threads_without_links_question_metadata))
threads_without_documentation_links_question_metadata$Category <- rep(1, nrow(threads_without_documentation_links_question_metadata))
threads_with_documentation_links_question_metadata$Category <- rep(2, nrow(threads_with_documentation_links_question_metadata))
merged_data <- data.frame(rbind(threads_without_links_question_metadata, threads_without_documentation_links_question_metadata, threads_with_documentation_links_question_metadata))
incomplete_cases <- which(is.na(merged_data$OwnerUserId))
length(incomplete_cases)
# 0
get_na_columns(merged_data)
# character(0)
write.table(merged_data, file=paste0(PREFIX, "/merged_data.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


###########################################################

### java_android ###

PREFIX <- "java_android"

# read sample of threads without links
threads_without_links <- read.csv(file=paste0(PREFIX, "/threads_without_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_without_links)
# 101
length(unique(threads_without_links$PostId))
# 101

# read sample of threads with documentation links
threads_with_documentation_links <- read.csv(file=paste0(PREFIX, "/threads_with_documentation_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_with_documentation_links)
# 101
length(unique(threads_with_documentation_links$PostId))
# 101

# read sample of threads without documentation links
threads_without_documentation_links <- read.csv(file=paste0(PREFIX, "/threads_without_documentation_links_100.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(threads_without_documentation_links)
# 105
length(unique(threads_without_documentation_links$PostId))
# 105

question_metadata <- retrieve_metadata()
question_post_blocks <- retrieve_postblocks()

# retrieve metadata for samples

# threads without links
threads_without_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_without_links$PostId,]
nrow(threads_without_links_question_metadata)
# 101
incomplete_cases <- which(is.na(threads_without_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 1
threads_without_links_question_metadata <- threads_without_links_question_metadata[-incomplete_cases,]
nrow(threads_without_links_question_metadata)
# 100
threads_without_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_without_links_question_metadata$PostId,]
nrow(threads_without_links_question_post_blocks)
# 440
# export data
write.table(threads_without_links_question_metadata, file=paste0(PREFIX, "/threads_without_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_without_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_without_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# threads with documentation links
threads_with_documentation_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_with_documentation_links$PostId,]
nrow(threads_with_documentation_links_question_metadata)
# 101
incomplete_cases <- which(is.na(threads_with_documentation_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 1
threads_with_documentation_links_question_metadata <- threads_with_documentation_links_question_metadata[-incomplete_cases,]
nrow(threads_with_documentation_links_question_metadata)
# 100
threads_with_documentation_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_with_documentation_links_question_metadata$PostId,]
nrow(threads_with_documentation_links_question_post_blocks)
# 330
# export data
write.table(threads_with_documentation_links_question_metadata, file=paste0(PREFIX, "/threads_with_documentation_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_with_documentation_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_with_documentation_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# threads without documentation links
threads_without_documentation_links_question_metadata <- question_metadata[question_metadata$PostId %in% threads_without_documentation_links$PostId,]
nrow(threads_without_documentation_links_question_metadata)
# 105
incomplete_cases <- which(is.na(threads_without_documentation_links_question_metadata$OwnerUserId))
length(incomplete_cases)
# 5
threads_without_documentation_links_question_metadata <- threads_without_documentation_links_question_metadata[-incomplete_cases,]
nrow(threads_without_documentation_links_question_metadata)
# 100
threads_without_documentation_links_question_post_blocks <- question_post_blocks[question_post_blocks$PostId %in% threads_without_documentation_links_question_metadata$PostId,]
nrow(threads_without_documentation_links_question_post_blocks)
# 414
# export data
write.table(threads_without_documentation_links_question_metadata, file=paste0(PREFIX, "/threads_without_documentation_links_question_metadata.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
write.table(threads_without_documentation_links_question_post_blocks[,c("PostId", "PostHistoryId", "LocalId", "Length", "LineCount", "ContentText")], file=paste0(PREFIX, "/threads_without_documentation_links_question_post_blocks.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# merge metadata of all three samples 
threads_without_links_question_metadata$Category <- rep(0, nrow(threads_without_links_question_metadata))
threads_without_documentation_links_question_metadata$Category <- rep(1, nrow(threads_without_documentation_links_question_metadata))
threads_with_documentation_links_question_metadata$Category <- rep(2, nrow(threads_with_documentation_links_question_metadata))
merged_data <- data.frame(rbind(threads_without_links_question_metadata, threads_without_documentation_links_question_metadata, threads_with_documentation_links_question_metadata))
incomplete_cases <- which(is.na(merged_data$OwnerUserId))
length(incomplete_cases)
# 0
get_na_columns(merged_data)
# character(0)
write.table(merged_data, file=paste0(PREFIX, "/merged_data.csv"), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")
