# set working directory (see https://stackoverflow.com/a/35842119)
dir = tryCatch({
  # script being sourced
  getSrcDirectory()[1]
}, error = function(e) {
  # script being run in RStudio
  dirname(rstudioapi::getActiveDocumentContext()$path)
})
setwd(dir)

source("../global.R")

library(data.table)
library(devtools)
install_github('software-analytics/Rnalytica')
library(Rnalytica)


### java_regex ###

PREFIX <- "java_regex"

numeric_data <- read.csv(file=paste0(PREFIX, "/numeric_data.csv"), header=TRUE, sep=",", quote="\"", strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"), stringsAsFactors=FALSE)
nrow(numeric_data)
# 300

# exclude unnecessary variables and variable with only one value
numeric_data <- numeric_data[,-which(names(numeric_data) %in% c("PostId", "PostTypeId", "PostHistoryId", "OwnerUserId", "tag_regex", "tag_java"))]

# exclude column added due to traling comma in CSV file
numeric_data <- numeric_data[,1:length(numeric_data)-1]

get_na_columns(numeric_data)
# character(0)

plotVarClus(dataset=numeric_data, names(numeric_data), correlation.threshold=0.7)

correlations = get.vc(numeric_data, names(numeric_data))
correlations$sim
# assuming that Category is the third column
selected_metrics = AutoSpearman(dataset=numeric_data, names(numeric_data)[-3], spearman.threshold=0.7, verbose=TRUE)
#selected_metrics <- names(numeric_data)

model = glm(as.formula(paste('Category', '~', paste0(selected_metrics, collapse = '+'))), family=binomial(), data=numeric_data)
model
summary(model)

1 - (model$deviance/model$null.deviance) #should be at least 0.7, but could probably report otherwise

Anova(model, type=2, test.statistic = 'LR')

#data[,6] = factor(data[,6])
#fit(data, 'attracts_link', names(data[,-6]), classifier = 'lr', validation.params = list(boot.n = 2)) #uses bootstrap by default
#table(data[,6])
